package com.blazinggames.gamejam.data
{
	public class TileMap
	{
// ***** VARIABLES *****
		
		private var _width:int;
		private var _height:int;
		protected var _mapData:Array;
		protected var _mapChanged:Boolean;
		protected var _shouldWrap:Boolean;
		protected var _outOfBoundsValue:int;
		
// ***** CONSTRUCTOR *****
		
		public function TileMap(w:int = 64, h:int = 64, clearID:int = 0)
		{
			var cntrRow:int, cntrCol:int;
			
			_width = w;
			_height = h;
			_mapData = new Array(_height);
			for (cntrRow = 0; cntrRow < _height; ++cntrRow) {
				_mapData[cntrRow] = new Array(_width);
				for (cntrCol = 0; cntrCol < _width; ++cntrCol) {
					_mapData[cntrRow][cntrCol] = clearID;
				}
			}
			_shouldWrap = true;
			_outOfBoundsValue = clearID;
		}
	
// ***** METHODS - Map Management *****
		
		/**
		 * Gets the tile at the indicated location. If the location specified
		 * is outside the map, the wrap state will determine if the coordinates
		 * will wrap or if the outOfBounds value will be returned.
		 * 
		 * @param x map x coordinate
		 * @param y map y coordinate
		 * @return the tile at the indicated location.
		 */		
		public function getTile(x:int, y:int):int
		{
			if (_shouldWrap) {
				var tx:int = x % _width;
				if (tx < 0) tx += _width;
				var ty:int = y % _height;
				if (ty < 0) ty += _height;
				
				return _mapData[ty][tx];
			} else if ((x < 0) || (x >= _width) || (y < 0) || (y >= _height) )
				return _outOfBoundsValue;
			else
				return _mapData[y][x];
		}
		
		/**
		 * Sets the indicated tile to indicated value. If coordinates 
		 * specified are out of bounds, the request is IGNORED! While this
		 * could be wrapped, I feel too much danger of accidental wrapping.
		 * 
		 * @param x map x coordinate
		 * @param y map y coordinate
		 * @param value value to put into map
		 */		
		public function setTile(x:int, y:int, value:int, attribute:Object = null):void
		{
			if ((x<0) || (y<0) || (x>_width) ||(y>_height)) {
				throw new Error("INVALID COORDINATES IN TILEMAP (SimpleTileMap::setTile("+x+","+y+","+value+")");
				return;
			}
			_mapData[y][x] = value;
			_mapChanged = true;
		}
		
		/**
		 * Clears the map with the indicated value.
		 * 
		 * @param value value to clear the map with.
		 */
		public function clearMap(value:int):void
		{
			var cntrRow:int, cntrCol:int;
			
			for (cntrRow = 0; cntrRow < _height; ++cntrRow)
				for (cntrCol = 0; cntrCol < _width; ++cntrCol)
					_mapData[cntrRow][cntrCol] = value;
			_mapChanged = true;
		}
		
		public function setTilesFromMapData(data:Array, width:int, height:int, offsetX:int = 0, offsetY:int = 0):void
		{
			if ((offsetX < 0) || (offsetY < 0))
				return;
			var w:int = Math.min(width, _width - offsetX);
			var h:int = Math.min(height, _height - offsetY);
			var indx:int = 0;
			for (var cntrY:int = 0; cntrY < h; ++cntrY) {
				for (var cntrX:int = 0; cntrX < w; ++cntrX) {
					_mapData[cntrY + offsetY][cntrX + offsetX] = data[indx];
					++indx;
				}
			}
			_mapChanged = true;
		}
		
		public function createClip(x:int, y:int, w:int, h:int):TileMap
		{
			var clip:TileMap = new TileMap(w,h);
			for (var cntrRow:int = 0; cntrRow < h; ++cntrRow)
				for (var cntrCol:int = 0; cntrCol < w; ++cntrCol)
					clip.setTile(cntrCol, cntrRow, getTile(x + cntrCol, y + cntrRow));
			return clip;
		}
		
		public function pasteClip(clip:TileMap, x:int, y:int, w:int = 0, h:int = 0):void
		{
			var clipW:int = w;
			if (clipW < 1)
				clipW = clip.width;
			var clipH:int = h;
			if (clipH < 1)
				clipH = clip.height;
			for (var cntrY:int = 0; cntrY < clipH; ++cntrY)
				for (var cntrX:int = 0; cntrX < clipW; ++cntrX)
					setTile(x + cntrX, y+cntrY, clip.getTile(cntrX, cntrY));			
		}

		public function fillRect(x:int, y:int, w:int, h:int, value:int):void
		{
			for (var cntrY:int = 0; cntrY < h; ++cntrY)
				for (var cntrX:int = 0; cntrX < w; ++cntrX)
					setTile(x + cntrX, y+cntrY, value);
		}

// ***** METHODS - Getters and Setters *****
		
		public function get width():int
		{
			return _width;
		}

		public function set width(value:int):void
		{
			_width = value;
		}

		public function get height():int
		{
			return _height;
		}

		public function set height(value:int):void
		{
			_height = value;
		}

		public function get mapChanged():Boolean
		{
			return _mapChanged;
		}

		public function set mapChanged(value:Boolean):void
		{
			_mapChanged = value;
		}

		public function get shouldWrap():Boolean
		{
			return _shouldWrap;
		}

		public function set shouldWrap(value:Boolean):void
		{
			_shouldWrap = value;
		}

		public function get outOfBoundsValue():int
		{
			return _outOfBoundsValue;
		}

		public function set outOfBoundsValue(value:int):void
		{
			_outOfBoundsValue = value;
		}


	}
}